@@ -216,6 +216,8 @@ class Projection(ProjectionBase):
 
     dependencies = ['clon', 'clat']
 
+    connections = ['transform']
+
     def __init__(self, *args, **kwargs):
         super(Projection, self).__init__(*args, **kwargs)
         self.projection = None
@@ -227,6 +229,8 @@ class Projection(ProjectionBase):
         if self.plotter.cleared:
             self.ax.projection = self.projection
             self.ax.clear()
+        if self.transform.value == value:
+            self.transform.projection = self.projection
 
     def update(self, value):
         """Update the formatoption
@@ -510,10 +514,13 @@ class LonLatBox(BoxBase):
             if is_rectilinear:
                 kwargs = dict(zip(
                     data.dims[-2:], starmap(slice, [lat_values, value[:2]])))
-                return data.sel(**kwargs)
+                ret = data.sel(**kwargs)
             else:
-                return self.mask_outside(data.copy(True), lon, lat, *value,
-                                         is_unstructured=is_unstructured)
+                ret = self.mask_outside(data.copy(True), lon, lat, *value,
+                                        is_unstructured=is_unstructured)
+            lon, lat = self._get_lola(ret, decoder)
+            self.data_lonlatbox = self.calc_lonlatbox(lon, lat)
+            return ret
 
     def to_degree(self, units=None, *args):
         """Converts arrays with radian units to degree
@@ -544,14 +551,23 @@ class LonLatBox(BoxBase):
                      is_unstructured=False):
         data.values = data.values.copy()
         ndim = 2 if not is_unstructured else 1
+        if lonmax < lonmin:
+            lonmax += 360
+            lon[lon < 0] += 360
+        mask = np.any([lon < lonmin, lon > lonmax, lat < latmin,
+                       lat > latmax], axis=0)
         if data.ndim > ndim:
-            for i, arr in enumerate(data.values):
-                arr[np.any([lon < lonmin, lon > lonmax, lat < latmin,
-                            lat > latmax], axis=0)] = np.nan
-                data.values[i, :] = arr
+            if is_unstructured:
+                data = data.psy[:, np.where(~mask)[0]]
+            else:
+                for i, arr in enumerate(data.values):
+                    arr[mask] = np.nan
+                    data.values[i, :] = arr
         else:
-            data.values[np.any([lon < lonmin, lon > lonmax, lat < latmin,
-                                lat > latmax], axis=0)] = np.nan
+            if is_unstructured:
+                data = data.psy[np.where(~mask)[0]]
+            else:
+                data.values[mask] = np.nan
         return data
 
     def calc_lonlatbox(self, lon, lat):
@@ -753,7 +769,10 @@ class Transform(ProjectionBase):
     connections = ['plot', 'vplot']
 
     def update(self, value):
-        self.projection = self.set_projection(value, 0, 0)
+        if value == 'cyl':
+            self.projection = ccrs.PlateCarree()
+        else:
+            self.projection = self.set_projection(value, 0, 0)
         for key in self.connections:
             try:
                 getattr(self, key)._kwargs['transform'] = self.projection
@@ -965,13 +984,10 @@ class GridLabelSize(Formatoption):
                 gl = fmto._gridliner
             except AttributeError:
                 continue
-            if self.plotter._initializing or self.plotter.has_changed(
-                    fmto.key):
-                gl.xlabel_style['size'] = value
-                gl.ylabel_style['size'] = value
-            else:
-                for text in chain(gl.xlabel_artists, gl.ylabel_artists):
-                    text.set_size(value)
+            gl.xlabel_style['size'] = value
+            gl.ylabel_style['size'] = value
+            for text in chain(gl.xlabel_artists, gl.ylabel_artists):
+                text.set_size(value)
 
 
 @docstrings.get_sectionsf('GridBase', sections=['Possible types', 'See Also'])
@@ -1175,64 +1191,91 @@ class MapPlot2D(psyps.Plot2D):
                           'correctly warped around!')
         return super(MapPlot2D, self)._contourf()
 
-    def _tripcolor(self):
-        from matplotlib.tri import Triangulation
-        self.logger.debug("Getting triangles")
-        triangles = self.triangles
-        triangles_wrapped = None
+    @property
+    def cell_nodes_x(self):
+        xbounds = super(MapPlot2D, self).cell_nodes_x
         if isinstance(self.transform.projection, ccrs.PlateCarree):
-            xbounds = triangles.x[triangles.triangles]
-            diffs = np.c_[np.diff(xbounds), np.diff(xbounds[:, ::-2])]
-            mask = (np.abs(diffs) > 180).any(axis=1)
-            x_wrap = xbounds[mask]
-            if x_wrap.size:
-                y_wrap = triangles.y[triangles.triangles[mask]]
-                diffs = diffs[mask]
-                x_wrap[diffs < -180] -= 360
-                diffs = np.c_[np.diff(x_wrap), np.diff(x_wrap[:, ::-2])]
-                x_wrap[diffs < -180] -= 360
-                triangles_wrapped = Triangulation(
-                    x_wrap.ravel(), y_wrap.ravel(),
-                    triangles=np.arange(x_wrap.size).reshape(x_wrap.shape))
-                # we have to apply a little workaround here in order to draw
-                # the  boundaries right. That implies that we mask out flat
-                # triangles (epecially those at the end) and transform them
-                # manually
-                triangles.set_mask(mask)
-        arr = None
-        try:
-            N = self.bounds.norm.Ncmap
-        except AttributeError:
-            arr = self.array
-            N = len(np.unique(self.bounds.norm(arr.ravel())))
-        cmap = psyps.get_cmap(self.cmap.value, N)
+            xbounds = xbounds.copy()
+            xbounds[xbounds - xbounds.min(axis=1, keepdims=True) > 180] -= 360
+        return xbounds
+
+    def _polycolor(self):
+        from matplotlib.collections import PolyCollection
+        self.logger.debug('Retrieving data')
+        arr = self.array
+        cmap = self.cmap.get_cmap(arr)
         if hasattr(self, '_plot'):
-            self.logger.debug("Updating plot")
+            self.logger.debug('Updating plot')
             self._plot.update(dict(cmap=cmap, norm=self.bounds.norm))
+            if hasattr(self, '_wrapped_plot'):
+                self._wrapped_plot.update(dict(cmap=cmap,
+                                               norm=self.bounds.norm))
         else:
-            self.logger.debug("Creating new plot")
-            if arr is None:
-                arr = self.array
-            arr_plot = arr[~np.isnan(arr)]
-            self.logger.debug("Creating %i triangles", len(arr_plot))
-            self._plot = self.ax.tripcolor(
-                triangles, arr_plot, norm=self.bounds.norm, cmap=cmap,
-                rasterized=True, **self._kwargs)
-        # draw wrapped collection to fix the issue that the boundaries are
-        # masked out when using the min_circle_ration
-        if triangles_wrapped and not hasattr(self, '_wrapped_plot'):
-            self.logger.debug("Creating new wrapped plot")
-            arr_wrap = arr_plot[mask]
-            kwargs = self._kwargs.copy()
-            kwargs['zorder'] = self._plot.zorder - 0.1
-            self._wrapped_plot = self.ax.tripcolor(
-                triangles_wrapped, arr_wrap, norm=self.bounds.norm,
-                cmap=cmap, rasterized=True, **kwargs)
-        elif hasattr(self, '_wrapped_plot'):
-            self.logger.debug("Updating wrapped plot")
-            self._wrapped_plot.update(dict(cmap=cmap, norm=self.bounds.norm))
-        self.logger.debug("Plot made. Done.")
-        return
+            self.logger.debug('Retrieving bounds')
+            xb = self.cell_nodes_x
+            yb = self.cell_nodes_y
+            wrap_proj_types = (ccrs._RectangularProjection,
+                               ccrs._WarpedRectangularProjection,
+                               ccrs.InterruptedGoodeHomolosine,
+                               ccrs.Mercator)
+            t = self.transform.projection
+            proj = self.ax.projection
+            wrapped_arr = None
+            ### HACK: since the matplotlib transformation is very slow because
+            # it transforms every single polygon, we transform the coordinates
+            # already here and do not specify the `transform` property for the
+            # `PolyCollection` down below.
+            # However, in order to wrap the boundaries correctly, we have to
+            # identify the corresponding grid cells and then use the standard
+            # matplotlib transform.
+            if isinstance(t, wrap_proj_types) and 'lon_0' in proj.proj4_params:
+                # First we transform the coordinates to a PlateCarree
+                # projection with the same center longitude as the projection
+                # of the plot.
+                # By using the geodetic version of `t`, we place the center
+                # longitude to `lon_0` and cells at the boundary do have a
+                # distance of 180 degrees to lon_0
+                lon_0 = proj.proj4_params['lon_0']
+                xb_trans = ccrs.PlateCarree(lon_0).transform_points(
+                    t.as_geodetic(), xb, yb)[..., 0].reshape(xb.shape)
+                # now we identify the cells at the x-boundary by
+                # checking whether they have coordinates close to 180 and
+                # close to -180
+                mask = np.any(
+                    xb_trans - xb_trans.min(axis=1, keepdims=True) > 180,
+                    axis=1)
+                # if so, we create a wrapped collection that is transformed in
+                # the standard matplotlib way.
+                if mask.any():
+                    wrapped_arr = arr[mask]
+                    arr = arr[~mask]
+                    xb_wrap = xb[mask]
+                    yb_wrap = yb[mask]
+                    xb = xb[~mask]
+                    yb = yb[~mask]
+            self.logger.debug('Making plot with %i cells', arr.size)
+            transformed = proj.transform_points(
+                t.as_geodetic(), xb.ravel(), yb.ravel())[..., :2].reshape(
+                    xb.shape + (2, ))
+            self._plot = PolyCollection(
+                transformed, array=arr.ravel(),
+                norm=self.bounds.norm, rasterized=True, cmap=cmap,
+                edgecolors='none', antialiaseds=False)
+            self.logger.debug('Adding collection to axes')
+            self.ax.add_collection(self._plot)
+            self._plot.set_clip_path(self.ax.outline_patch)
+            if wrapped_arr is not None:
+                self.logger.debug('Making wrapped plot with %i cells',
+                                  wrapped_arr.size)
+                self._wrapped_plot = PolyCollection(
+                    np.dstack([xb_wrap, yb_wrap]), array=wrapped_arr.ravel(),
+                    norm=self.bounds.norm, rasterized=True, cmap=cmap,
+                    transform=t, zorder=self._plot.zorder - 0.1,
+                    edgecolors='none', antialiaseds=False)
+                self.logger.debug('Adding wrapped collection to axes')
+                self.ax.add_collection(self._wrapped_plot)
+                self._wrapped_plot.set_clip_path(self.ax.outline_patch)
+        self.logger.debug('Done.')
 
     def remove(self, *args, **kwargs):
         super(MapPlot2D, self).remove(*args, **kwargs)
@@ -1261,39 +1304,57 @@ class MapDataGrid(psyps.DataGrid):
     xgrid
     ygrid""")
 
-    def triangles(self):
-        from matplotlib.tri import TriAnalyzer
-        decoder = self.decoder
-        triangles = decoder.get_triangles(
-            self.data, self.data.coords, copy=True,
-            src_crs=self.transform.projection, target_crs=self.ax.projection)
-        mratio = rcParams['plotter.plot2d.plot.min_circle_ratio']
-        if mratio:
-            triangles.set_mask(
-                TriAnalyzer(triangles).get_flat_tri_mask(mratio))
-        triangles.set_mask(np.isnan(
-            self.data.values if self.data.ndim == 1 else self.data[0].values))
-        # in order to avoid lines spanning from right to left over the whole
-        # plot due to the warping in cylindric (or quasi rectangular)
-        # projections, we mask those triangles on the left and right side of
-        # the globe
-        wrap_proj_types = (ccrs._RectangularProjection,
-                           ccrs._WarpedRectangularProjection,
-                           ccrs.InterruptedGoodeHomolosine,
-                           ccrs.Mercator)
-        if isinstance(self.ax.projection, wrap_proj_types):
-            lon_0 = self.ax.projection.proj4_params['lon_0']
-            xmin = lon_0 - 100.
-            xmax = lon_0 + 100.
-            x = ccrs.PlateCarree(lon_0).transform_points(
-                self.ax.projection, triangles.x, triangles.y)[:, 0]
-            x = x[triangles.triangles]
-            mask = np.array([
-                np.any(arr > xmax) and np.any(arr < xmin) for arr in x])
-            triangles.set_mask(mask)
-        return triangles
-
-    triangles = property(triangles, doc=psyps.DataGrid.triangles.__doc__)
+    def update(self, value):
+        self.remove()
+        if value is None:
+            return
+        xb = self.cell_nodes_x
+        yb = self.cell_nodes_y
+        if isinstance(self.transform.projection, ccrs.PlateCarree):
+            xb[xb - xb.min(axis=1, keepdims=True) > 180] -= 360
+        t = self.transform.projection
+        proj = self.ax.projection
+        # HACK: We remove the cells at the boundary of the map projection
+        xb_wrap = None
+        if isinstance(t, wrap_proj_types) and 'lon_0' in proj.proj4_params:
+            # See the :meth:`MapPlot2D._polycolor` method for a documentation
+            # of the steps
+            lon_0 = proj.proj4_params['lon_0']
+            xb_trans = ccrs.PlateCarree(lon_0).transform_points(
+                t.as_geodetic(), xb, yb)[..., 0].reshape(xb.shape)
+            mask = np.any(
+                xb_trans - xb_trans.min(axis=1, keepdims=True) > 180,
+                axis=1)
+            if mask.any():
+                xb_wrap = xb[mask]
+                yb_wrap = yb[mask]
+                xb = xb[~mask]
+                yb = yb[~mask]
+        orig_shape = xb.shape
+        transformed = proj.transform_points(t.as_geodetic(), xb.ravel(),
+                                            yb.ravel())
+        xb = transformed[..., 0].reshape(orig_shape)
+        yb = transformed[..., 1].reshape(orig_shape)
+        # We insert nan values in the flattened edges arrays rather than
+        # plotting the grid cells separately as it considerably speeds-up code
+        # execution.
+        n = len(xb)
+        xb = np.c_[xb, xb[:, :1], [[np.nan]] * n].ravel()
+        yb = np.c_[yb, yb[:, :1], [[np.nan]] * n].ravel()
+        if isinstance(value, dict):
+            self._artists = self.ax.plot(xb, yb, **value)
+        else:
+            self._artists = self.ax.plot(xb, yb, value)
+        if xb_wrap is not None:
+            n = len(xb_wrap)
+            xb_wrap = np.c_[xb_wrap, xb_wrap[:, :1], [[np.nan]] * n].ravel()
+            yb_wrap = np.c_[yb_wrap, yb_wrap[:, :1], [[np.nan]] * n].ravel()
+            if isinstance(value, dict):
+                self._artists.extend(self.ax.plot(xb_wrap, yb_wrap,
+                                                  transform=t, **value))
+            else:
+                self._artists.extend(self.ax.plot(xb_wrap, yb_wrap, value,
+                                                  transform=t))
 
 
 class MapDensity(psyps.Density):
